﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.Diagnostics;

namespace howto_plinq_primes
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Generate a list of numbers and pick out the prime ones.
        private void btnGo_Click(object sender, EventArgs e)
        {
            lstAllNumbers.DataSource = null;
            lstLinq.DataSource = null;
            lstPlinq.DataSource = null;
            lblLinq.Text = "";
            lblPlinq.Text = "";
            Cursor = Cursors.WaitCursor;
            Refresh();
            Stopwatch watch = new Stopwatch();

            // Create the numbers.
            watch.Start();
            Random rand = new Random();
            int num_numbers = int.Parse(txtNumNumbers.Text);
            int[] numbers = new int[num_numbers];
            for (int i = 0; i < numbers.Length; i++)
                numbers[i] = rand.Next(0, 100);
            watch.Stop();
            lstAllNumbers.DataSource = numbers.Take(1000).ToArray();
            lblRandom.Text = watch.Elapsed.TotalSeconds.ToString("0.00") + " secs";
            lblLinq.Refresh();
            lstAllNumbers.Refresh();

            // Make an IsPrime delegate.
            Func<int, bool> IsPrime = number =>
            {
                if (number == 0) return false;
                else if (number < 0) number = -number;

                if (number == 1) return false;
                if (number == 2) return true;
                if (number % 2 == 0) return false;
                for (int i = 3; i * i <= number; i += 2)
                    if (number % i == 0) return false;
                return true;
            };

            // Use LINQ to find the prime numbers.
            watch.Restart();
            var linq_query =
                from int number in numbers
                where IsPrime(number)
                select number;
            int[] linq_primes = linq_query.ToArray();
            watch.Stop();
            lstLinq.DataSource = linq_primes.Take(1000).ToArray();
            lblLinq.Text = watch.Elapsed.TotalSeconds.ToString("0.00") + " secs";
            lblLinq.Refresh();
            lstLinq.Refresh();

            // Use PLINQ to find the prime numbers.
            watch.Restart();
            var plinq_query =
                from int number in numbers.AsParallel()
                where IsPrime(number)
                select number;
            int[] plinq_primes = plinq_query.ToArray();
            watch.Stop();
            lstPlinq.DataSource = plinq_primes.Take(1000).ToArray();
            lblPlinq.Text = watch.Elapsed.TotalSeconds.ToString("0.00") + " secs";

            Cursor = Cursors.Default;
        }
    }
}
